import pytest


def test_001_i_want_to_run_specific_test(admlc, admlf):

  # Send POST (test flow with a single step changing ip-limit to 15)
  flow = admlf.getId()
  response = admlc.postDict("/testid-ip-limit/{}".format(flow), { "amount":15 })

  # Verify response (test & step programmed)
  responseBodyRef = { "success":"true", "response":"Done" }
  admlc.assert_response__status_body_headers(response, 200, responseBodyRef)

  # Send POST (execute the former test)
  response = admlc.postDict("/test-run", { "id":flow })

  # Verify response
  responseBodyRef = { "success":"true", "response":"Test executed for id provided ({})".format(flow) }
  admlc.assert_response__status_body_headers(response, 200, responseBodyRef)

  # Verify ip-limit now:
  response = admlc.postDict("/test-ip-limit", {}) # default {} to query current state
  responseBodyRef = { "success":"true", "response":"In-progress limit amount: 15; currently there are 0 test cases running" }
  admlc.assert_response__status_body_headers(response, 200, responseBodyRef)

  # Cleanup & restore ip-limit (at global level because it is easier)
  response = admlc.post("/test-clear")
  response = admlc.postDict("/test-ip-limit", { "amount":-1 })


def test_002_i_want_to_run_unexisting_test(admlc):

  # Send POST
  response = admlc.postDict("/test-run", { "id":666 })

  # Verify response
  responseBodyRef = { "success":"false", "response":"Internal error (check ADML traces): cannot found test id (666)" }
  admlc.assert_response__status_body_headers(response, 200, responseBodyRef)

