// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#ifndef anna_comm_DirectTransport_hpp
#define anna_comm_DirectTransport_hpp

#include <anna/core/DataBlock.hpp>

#include <anna/comm/Transport.hpp>
#include <anna/comm/TransportFactory.hpp>
#include <anna/comm/TransportFactoryImpl.hpp>

namespace anna {

namespace comm {

/**
   Clase generica para definir la capa de transporte del protocolo de comunicaciones sin ningun tipo
   de aditivos, envia el bloque de datos tal y como le llega.

   Este protocolo esta orientado a intercambiar mensajes en una red interna, por lo que supone
   que no van a existir errores en los mensajes recibidos y que los mensajes son suficientemente cortos
   como poder representar su longitud en dos unicos bytes.

   Los supuestos bajo los que se puede este protocolo facilitan el desarrollo de clases que ofrecen un
   gran rendimiento, pero imposibilitan el desarrollo del sistema de re-sincronizacin en caso de que alguno
   de los mensajes no cumpla los supuestos. Es decir, si nos llega un mensaje erroneo nuestro proceso no sera
   capaz de volver a sincronizarse nunca mas y terminara cerrando el ClientSocket.

   \warning Hay que tener en cuenta que el metodo de codificacion no ofrece informacin
   suficiente para poder re-sincronizar los procesos en caso de error.

   \see Transport.
*/
class DirectTransport : public comm::Transport {
public:
  /**
     Destructor
  */
  virtual ~DirectTransport();

  /**
     Devuelve el gestor de capas de transporte asociado a esta clase.
     \return El gestor de capas de transporte asociado a esta clase.
  */
  static TransportFactory& getFactory() throw() { return st_transportFactory; }

  /**
      Devuelve el literal que indentifica de esta clase.
      \return el literal que indentifica de esta clase.
   */
  static const char* className() throw() { return "anna::comm::DirectTransport"; }

private:
  static TransportFactoryImpl <DirectTransport> st_transportFactory;

  DirectTransport();

  int calculeSize(const DataBlock& dataBlock) throw(RuntimeException) { return dataBlock.getSize(); }
  const Message* decode(const DataBlock&) throw(RuntimeException);
  const DataBlock& code(Message&) throw(RuntimeException);

  friend class anna::Allocator <DirectTransport>;
};

}
}
#endif

