// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <anna/core/util/Component.hpp>
#include <anna/core/util/ComponentManager.hpp>

#include <anna/core/RuntimeException.hpp>
#include <anna/core/functions.hpp>
#include <anna/xml/Node.hpp>
#include <anna/xml/Attribute.hpp>
#include <anna/core/tracing/Logger.hpp>


anna::Component::Component(const char* className) :
  a_className(className) {
  ComponentManager &cm = ComponentManager::instantiate();

  try {
    cm.attach(this);
  } catch(anna::RuntimeException& ex) {
    ex.trace();
  }
}

anna::Component::~Component() {
  ComponentManager &cm = ComponentManager::instantiate();

  try {
    cm.detach(this);
  } catch(anna::RuntimeException& ex) {
    ex.trace();
  }
}

std::string anna::Component::asString() const
throw() {
  std::string result("anna::Component { Name: ");
  result += a_className;
  result += " | Reference: ";
  result += anna::functions::asHexString(anna_ptrnumber_cast(this));
  return result += " }";
}

anna::xml::Node* anna::Component::asXML(anna::xml::Node* parent) const
throw() {
  anna::xml::Node* result = parent->createChild("anna.Component");
  result->createAttribute("Name", a_className);
  result->createAttribute("Reference", anna::functions::asHexString(anna_ptrnumber_cast(this)));
  return result;
}

