// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


// Local
#include <anna/diameter/codec/functions.hpp>
#include <anna/diameter/codec/Message.hpp>
#include <anna/diameter/codec/Avp.hpp>
#include <anna/diameter/defines.hpp>
#include <anna/diameter/functions.hpp>
#include <anna/config/defines.hpp> // general types, decoding helpers (DECODE[2/3/4]BYTES_INDX_VALUETYPE), etc.

#include <anna/core/DataBlock.hpp>
#include <anna/core/functions.hpp>
#include <anna/core/tracing/Logger.hpp>

// STL
#include <string>



namespace anna {
namespace diameter {
namespace codec {
// Preloaded MemoryDTD for function helpers:
anna::xml::DTDMemory MessageDTDMemory(MessageDTD);
}
}
}


// Parent struct helper /////////////////////////////////////////////////////////////////////////////
void anna::diameter::codec::parent::setMessage(const anna::diameter::CommandId & mid, const char *mname) throw() {
  MessageId = mid;
  if (mname) {
    MessageName = mname;
  }
  else {
    MessageName = "Message";
    MessageName += anna::diameter::functions::commandIdAsPairString(mid);
  }
}

void anna::diameter::codec::parent::addAvp(const anna::diameter::AvpId & aid, const char *aname) throw() {
  AvpsId.push_back(aid);
  std::string name;
  if (aname) {
    name = aname;
  }
  else {
    name = "Avp";
    name += anna::diameter::functions::avpIdAsPairString(aid);
  }
  AvpsName.push_back(name);
}

std::string anna::diameter::codec::parent::asString() const throw() { // "<command><avp 1>-><avp 2>->...-><avp N>"
  std::string result = MessageName;
  for (std::vector<std::string>::const_iterator it = AvpsName.begin(); it != AvpsName.end(); it++) {
    result += "->";
    result += (*it);
  }

  return result;
}
/////////////////////////////////////////////////////////////////////////////////////////////////////


// getters
anna::diameter::CommandId anna::diameter::codec::functions::getCommandId(const anna::DataBlock & db) throw(anna::RuntimeException) {
  if(db.getSize() < Message::HeaderLength)
    throw anna::RuntimeException("Not enough bytes to cover command header length", ANNA_FILE_LOCATION);

  const char * data = db.getData();
  U8 flags =  data[4];
  U24 code = DECODE3BYTES_INDX_VALUETYPE(data, 5, U24);
  //   U24 code = (((U24)data[5] << 16) & 0xFF0000) +
  //              (((U24)data[6] << 8)  & 0x00FF00) +
  //              (((U24)data[7]) & 0x0000FF);
  return (anna::diameter::CommandId(code, (flags & Message::RBitMask) != 0x00));
}

bool anna::diameter::codec::functions::requestBit(const anna::DataBlock & db) throw(anna::RuntimeException) {
  if(db.getSize() < Message::HeaderLength)
    throw anna::RuntimeException("Not enough bytes to cover command header length", ANNA_FILE_LOCATION);

  return (((db.getData())[4] & Message::RBitMask) != 0x00);
}

bool anna::diameter::codec::functions::proxiableBit(const anna::DataBlock & db) throw(anna::RuntimeException) {
  if(db.getSize() < Message::HeaderLength)
    throw anna::RuntimeException("Not enough bytes to cover command header length", ANNA_FILE_LOCATION);

  return (((db.getData())[4] & Message::PBitMask) != 0x00);
}

bool anna::diameter::codec::functions::errorBit(const anna::DataBlock & db) throw(anna::RuntimeException) {
  if(db.getSize() < Message::HeaderLength)
    throw anna::RuntimeException("Not enough bytes to cover command header length", ANNA_FILE_LOCATION);

  return (((db.getData())[4] & Message::EBitMask) != 0x00);
}

bool anna::diameter::codec::functions::potentiallyReTransmittedMessageBit(const anna::DataBlock & db) throw(anna::RuntimeException) {
  if(db.getSize() < Message::HeaderLength)
    throw anna::RuntimeException("Not enough bytes to cover command header length", ANNA_FILE_LOCATION);

  return (((db.getData())[4] & Message::TBitMask) != 0x00);
}

anna::diameter::ApplicationId anna::diameter::codec::functions::getApplicationId(const anna::DataBlock & db) throw(anna::RuntimeException) {
  if(db.getSize() < Message::HeaderLength)
    throw anna::RuntimeException("Not enough bytes to cover command header length", ANNA_FILE_LOCATION);

  const char * appidPtr = db.getData() + 8;
  anna::diameter::ApplicationId result = DECODE4BYTES_INDX_VALUETYPE(appidPtr, 0, U32);
  //   anna::diameter::ApplicationId result = (((U32)appidPtr[0] << 24) & 0xFF000000) +
  //                               (((U32)appidPtr[1] << 16) & 0x00FF0000) +
  //                               (((U32)appidPtr[2] << 8) & 0x0000FF00) +
  //                               (((U32)appidPtr[3]) & 0x000000FF);
  return result;
}

anna::diameter::HopByHop anna::diameter::codec::functions::getHopByHop(const anna::DataBlock & db) throw(anna::RuntimeException) {
  if(db.getSize() < Message::HeaderLength)
    throw anna::RuntimeException("Not enough bytes to cover command header length", ANNA_FILE_LOCATION);

  const char * hbhPtr = db.getData() + 12;
  anna::diameter::HopByHop result = DECODE4BYTES_INDX_VALUETYPE(hbhPtr, 0, U32);
  //   anna::diameter::HopByHop result = (((U32)hbhPtr[0] << 24) & 0xFF000000) +
  //                               (((U32)hbhPtr[1] << 16) & 0x00FF0000) +
  //                               (((U32)hbhPtr[2] << 8) & 0x0000FF00) +
  //                               (((U32)hbhPtr[3]) & 0x000000FF);
  return result;
}

anna::diameter::EndToEnd anna::diameter::codec::functions::getEndToEnd(const anna::DataBlock & db) throw(anna::RuntimeException) {
  if(db.getSize() < Message::HeaderLength)
    throw anna::RuntimeException("Not enough bytes to cover command header length", ANNA_FILE_LOCATION);

  const char * etePtr = db.getData() + 16;
  anna::diameter::EndToEnd result = DECODE4BYTES_INDX_VALUETYPE(etePtr, 0, U32);
  //   anna::diameter::EndToEnd result = (((U32)etePtr[0] << 24) & 0xFF000000) +
  //                               (((U32)etePtr[1] << 16) & 0x00FF0000) +
  //                               (((U32)etePtr[2] << 8) & 0x0000FF00) +
  //                               (((U32)etePtr[3]) & 0x000000FF);
  return result;
}

void anna::diameter::codec::functions::decodeCommandHeader(const char *start, char & version, U24 & length, char & flags, diameter::CommandId & id, int & appId, int & hbh, int & ete) throw(anna::RuntimeException) {
  if(start == NULL)
    throw anna::RuntimeException("NULL provided start pointer", ANNA_FILE_LOCATION);

  //      0                   1                   2                   3
  //      0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
  //      +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
  //      |    Version    |                 Message Length                |
  //      +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
  //      | command flags |                  Command-Code                 |
  //      +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
  //      |                         Application-ID                        |
  //      +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
  //      |                      Hop-by-Hop Identifier                    |
  //      +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
  //      |                      End-to-End Identifier                    |
  //      +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
  //      |  AVPs ...
  //      +-+-+-+-+-+-+-+-+-+-+-+-+-
  version = start[0]; // Version
  //length = (start[1] << 16) + (start[2] << 8) + start[3]; // Message Length
  length = DECODE3BYTES_INDX_VALUETYPE(start, 1, U24);
  flags = start[4]; // command flags
  //id.first = (start[5] << 16) + (start[6] << 8) + start[7]; // Command-Code
  id.first = DECODE3BYTES_INDX_VALUETYPE(start, 5, U24);
  id.second = ((flags & Message::RBitMask) != 0x00); // Command-Code is a request
  //appId = (start[8] << 24) + (start[9] << 16) + (start[10] << 8) + start[11]; // Application-ID
  appId = DECODE4BYTES_INDX_VALUETYPE(start, 8, U32);
  //hbh = (start[12] << 24) + (start[13] << 16) + (start[14] << 8) + start[15]; // Hop-by-Hop Identifier
  hbh = DECODE4BYTES_INDX_VALUETYPE(start, 12, U32);
  //ete = (start[16] << 24) + (start[17] << 16) + (start[18] << 8) + start[19]; // End-to-End Identifier
  ete = DECODE4BYTES_INDX_VALUETYPE(start, 16, U32);
}

void anna::diameter::codec::functions::decodeAVP(const char *start, diameter::AvpId & id, char & flags, int & length, std::string & data) throw(anna::RuntimeException) {
  if(start == NULL)
    throw anna::RuntimeException("NULL provided start pointer", ANNA_FILE_LOCATION);

  //       0                   1                   2                   3
  //       0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
  //      +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
  //      |                           AVP Code                            |
  //      +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
  //      |   AVP Flags   |                  AVP Length                   |
  //      +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
  //      |                        Vendor-ID (opt)                        |
  //      +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
  //      |    Data ...
  //      +-+-+-+-+-+-+-+-+
  //id.first = (start[0] << 24) + (start[1] << 16) + (start[2] << 8) + start[3]; // AVP Code
  id.first = DECODE4BYTES_INDX_VALUETYPE(start, 0, S32);
  flags = start[4]; // AVP Flags
  //length = (start[5] << 16) + (start[6] << 8) + start[7]; // AVP Length
  length = DECODE3BYTES_INDX_VALUETYPE(start, 5, U24);
  bool vendorSpecific = ((flags & Avp::VBitMask) != 0x00); // AVP Code is vendor-specific
  //id.second = (vendorSpecific ? ((start[8] << 24) + (start[9] << 16) + (start[10] << 8) + start[11]) : 0 /* IETF */); // Vendor-ID (opt)
  id.second = (vendorSpecific ? (DECODE4BYTES_INDX_VALUETYPE(start, 8, S32)) : 0 /* IETF */); // Vendor-ID (opt)
  int dataLength = (vendorSpecific ? (length - 12) : (length - 8)); // avp data part length
  const char *dataPointer = (vendorSpecific ? (start + 12) : (start + 8)); // pointer to data part
  data.assign(dataPointer, dataLength);
  LOGLOCAL3(
      std::string msg = anna::functions::asString("decodedAVP id (%d,%d), length %d, data length %d, data part 0x%s",
          id.first, id.second, length, dataLength, anna::functions::asHexString(anna::DataBlock(dataPointer, dataLength)).c_str());
  anna::Logger::write(anna::Logger::Local3, msg, ANNA_FILE_LOCATION);
  );
}

const char * anna::diameter::codec::functions::nextAVP(const char *avpsDB, int avpsLen, const char *start) throw(anna::RuntimeException) {
  if(start == NULL)
    throw anna::RuntimeException("NULL provided start pointer", ANNA_FILE_LOCATION);
  if(avpsDB == NULL)
    throw anna::RuntimeException("NULL provided avpsDB pointer", ANNA_FILE_LOCATION);

  const char *result;
  //   LOGDEBUG(
  //      std::string msg("DataBlock provided to 'nextAVP'");
  //      msg += avpsDB.asString();
  //      anna::Logger::debug(msg, ANNA_FILE_LOCATION);
  //   );
  //int avpLength = (start[5] << 16) + (start[6] << 8) + start[7]; // AVP Length
  int avpLength = DECODE3BYTES_INDX_VALUETYPE(start, 5, int);
  result = start + 4 * REQUIRED_WORDS(avpLength);
  int offset = (result - avpsDB);

  if(offset > (avpsLen - 1))
    //throw anna::RuntimeException("Start pointer out of boundaries for block(avpsDB, avpsLen)", ANNA_FILE_LOCATION);
    return NULL; // (*)

  return result;
}

//const char * functions::nextAVP(const anna::DataBlock & avpsDB, const char *start) throw(anna::RuntimeException) {
//  return nextAVP(avpsDB.getData(), avpsDB.getSize(), start);
//}

const char * anna::diameter::codec::functions::findAVP(const char *avpsDB, int avpsLen, const diameter::AvpId & id, int n) throw(anna::RuntimeException) {
  const char *result = avpsDB; // first avp
  int positives = 0;
  // Decoded avp information:
  diameter::AvpId _id;
  char _flags;
  int _length;
  std::string _data;
  decodeAVP(result, _id, _flags, _length, _data);

  if(_id == id) positives++;

  while((_id != id) || (positives != n)) {  // next search if not found or not ocurrence number reached
    result = nextAVP(avpsDB, avpsLen, result);

    if(result == NULL) {  // (*)
      LOGDEBUG(
          std::string msg = "AVP ";
      msg += anna::diameter::functions::avpIdAsPairString(id);
      msg += " not found at DataBlock";
      anna::Logger::debug(msg, ANNA_FILE_LOCATION);
      );
      return NULL;
    }

    decodeAVP(result, _id, _flags, _length, _data);

    if(_id == id) positives++;
  }

  return result;
}

//const char * functions::findAVP(const anna::DataBlock & avpsDB, const diameter::AvpId & id, int n) throw(anna::RuntimeException) {
//  return findAVP(avpsDB.getData(), avpsDB.getSize(), id, n);
//}

// modifiers
void anna::diameter::codec::functions::setHopByHop(anna::DataBlock & db, diameter::HopByHop hbh) throw(anna::RuntimeException) {
  if(db.getSize() < Message::HeaderLength) {
    throw anna::RuntimeException("Not enough bytes to cover command header length", ANNA_FILE_LOCATION);
  }

  char source[4];
  source[0] = (char)(hbh >> 24);
  source[1] = (char)(hbh >> 16);
  source[2] = (char)(hbh >> 8);
  source[3] = (char)hbh;
  memcpy((char*)(db.getData() + 12), source, 4);
}


void anna::diameter::codec::functions::setEndToEnd(anna::DataBlock & db, diameter::EndToEnd ete) throw(anna::RuntimeException) {
  if(db.getSize() < Message::HeaderLength) {
    throw anna::RuntimeException("Not enough bytes to cover command header length", ANNA_FILE_LOCATION);
  }

  char source[4];
  source[0] = (char)(ete >> 24);
  source[1] = (char)(ete >> 16);
  source[2] = (char)(ete >> 8);
  source[3] = (char)ete;
  memcpy((char *)(db.getData() + 16), source, 4);
}

void anna::diameter::codec::functions::setPotentiallyReTransmittedMessageBit(const anna::DataBlock & db, bool activate) throw(anna::RuntimeException) {
  if(db.getSize() < Message::HeaderLength) {
    throw anna::RuntimeException("Not enough bytes to cover command header length", ANNA_FILE_LOCATION);
  }

  char flags[1];
  flags[0] = *(db.getData() + 4);
  if(activate) flags[0] |= Message::TBitMask; else flags[0] &= (~Message::TBitMask);
  memcpy((char *)(db.getData() + 4), flags, 1);
}

// XML parsers for diameter messages ///////////////////////////////////////////////////////////////////////////
void anna::diameter::codec::functions::messageXmlDocumentFromXmlFile(anna::xml::DocumentFile &xmlDocument, const std::string & xmlPathFile) throw(anna::RuntimeException) {
  LOGDEBUG(anna::Logger::debug(anna::functions::asString("Parsing diameter message from xml file '%s' into xml document", xmlPathFile.c_str()), ANNA_FILE_LOCATION));
  xmlDocument.initialize(xmlPathFile.c_str()); // fail here is i/o error
  const anna::xml::Node *rootNode = xmlDocument.parse(MessageDTDMemory); // Parsing: fail here if xml violates dtd
  LOGDEBUG(
      std::string trace = "Parsing OK from XML file '";
  trace += xmlPathFile;
  trace += "':\n";
  trace += anna::xml::Compiler().apply(rootNode);
  anna::Logger::debug(trace, ANNA_FILE_LOCATION);
  );
}

void anna::diameter::codec::functions::messageXmlDocumentFromXmlString(anna::xml::DocumentMemory &xmlDocument, const std::string &xmlString) throw(anna::RuntimeException) {
  LOGDEBUG(anna::Logger::debug("Parsing diameter message from xml string representation into xml document", ANNA_FILE_LOCATION));
  xmlDocument.initialize(xmlString.c_str());
  const anna::xml::Node *rootNode = xmlDocument.parse(MessageDTDMemory); // Parsing: fail here if xml violates dtd
  LOGDEBUG(
      std::string trace = "Parsing OK from XML string representation '";
  trace += xmlString;
  trace += "':\n";
  trace += anna::xml::Compiler().apply(rootNode);
  anna::Logger::debug(trace, ANNA_FILE_LOCATION);
  );
}
