// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


//------------------------------------------------------------------------------
//-------------------------------------------------------- included header files
//------------------------------------------------------------------------------

// Standard
#include <stdlib.h>

// Local
#include <anna/diameter/stack/AvpRule.hpp>
#include <anna/diameter/stack/Avp.hpp>
#include <anna/diameter/stack/Format.hpp>
#include <anna/diameter/functions.hpp>
#include <anna/diameter/stack/Dictionary.hpp>

#include <anna/config/defines.hpp>
#include <anna/core/RuntimeException.hpp>
#include <anna/xml/xml.hpp>

anna_assign_enum(anna::diameter::stack::AvpRule::Presence) = { "Fixed", "Mandatory", "Optional", NULL /* list end indicator */};


//------------------------------------------------------------------------------
//----------------------------------------------------------- AvpRule::setQual()
//------------------------------------------------------------------------------
void anna::diameter::stack::AvpRule::setQual(const std::string & q) throw(anna::RuntimeException) {
  const char *asterisk = strstr(q.c_str(), "*");

  if((q != "") && (asterisk == NULL))
    throw anna::RuntimeException("Non-empty qualifier must contain '*'", ANNA_FILE_LOCATION);

  a_qual = q;
}


//------------------------------------------------------------------------------
//-------------------------------------------------------- AvpRule::getAvpName()
//------------------------------------------------------------------------------
std::string anna::diameter::stack::AvpRule::getAvpName(void) const throw() {
  const Avp * avp = a_dictionary->getAvp(a_avpId);
  return avp->getName();
}

//------------------------------------------------------------------------------
//------------------------------------------------------------- AvpRule::isAny()
//------------------------------------------------------------------------------
bool anna::diameter::stack::AvpRule::isAny(void) const throw() {
  const Avp * avp = a_dictionary->getAvp(a_avpId);
  const Format * format = a_dictionary->getFormat(avp->getFormatName());
  return format->isAny();
}


//------------------------------------------------------------------------------
//-------------------------------------------------------- AvpRule::getQualMin()
//------------------------------------------------------------------------------
int anna::diameter::stack::AvpRule::getQualMin(void) const throw() {
  if(a_qual == "") {
    if(isFixed() || isMandatory()) return 1;

    if(isOptional()) return 0;
  }

  // Asterisk location
  const char * c_qual = a_qual.c_str();
  int asterisk_pos = strstr(c_qual, "*") - c_qual;

  // '*', '*y'
  if(asterisk_pos == 0) return 0;

  // 'x*', 'x*y'
  std::string min = a_qual.substr(0, asterisk_pos);  // 'x'
  return (atoi(min.c_str()));
}



//------------------------------------------------------------------------------
//-------------------------------------------------------- AvpRule::getQualMax()
//------------------------------------------------------------------------------
int anna::diameter::stack::AvpRule::getQualMax(void) const throw() {
  if(a_qual == "") return 1;

  // Asterisk location
  const char * c_qual = a_qual.c_str();
  int asterisk_pos = strstr(c_qual, "*") - c_qual;

  // '*', 'x*'
  if(asterisk_pos == (a_qual.size() - 1)) return -1;  // inf

  // '*y', 'x*y'
  std::string max = a_qual.substr(asterisk_pos + 1, a_qual.size() - asterisk_pos - 1);  // 'y'
  return (atoi(max.c_str()));
}


//------------------------------------------------------------------------------
//---------------------------------------------------------- AvpRule::asString()
//------------------------------------------------------------------------------
std::string anna::diameter::stack::AvpRule::asString(bool showPair) const throw() {
  std::string trace = "No Avp rule defined";
  const Avp * avp = a_dictionary->getAvp(a_avpId);

  if(avp) {
    trace = a_qual;
    std::string s_open, s_close;

    if(isFixed()) { s_open = "<"; s_close = ">"; }

    if(isMandatory()) { s_open = "{"; s_close = "}"; }

    if(isOptional()) { s_open = "["; s_close = "]"; }

    trace += s_open;
    trace += getAvpName(); // anna::diameter::functions::avpIdAsPairString(a_avpId);;
    trace += s_close;

    if(!showPair) return trace;

    // Avoid ambiguous descriptions:
    int qualSize = a_qual.size();
    int tabSize = strlen(DICTIONARY_AVPRULE_TAB);
    int NumberOfDots =   /* qual add */ ((qualSize > tabSize) ? tabSize : qualSize) +
                                        /* max expected avp description size */   48 -
                                        /* current trace length */                trace.size();

    for(int k = 0; k < NumberOfDots; k++) trace += ".";

    trace += anna::diameter::functions::avpIdAsPairString(avp->getId());
  }

  return (trace);
}


//------------------------------------------------------------------------------
//------------------------------------------------------------- AvpRule::asXML()
//------------------------------------------------------------------------------
anna::xml::Node* anna::diameter::stack::AvpRule::asXML(anna::xml::Node* parent) const throw() {
//   <!ELEMENT avprule EMPTY>
//   <!ATTLIST avprule id CDATA #REQUIRED type (Fixed | Mandatory | Optional) #REQUIRED qual CDATA #IMPLIED>
  anna::xml::Node* result = parent->createChild("avprule");
  result->createAttribute("id", getAvpName());
  std::string type;

  if(isFixed()) type = "Fixed";
  else if(isMandatory()) type = "Mandatory";
  else if(isOptional()) type = "Optional";

  result->createAttribute("type", type);

  if(a_qual != "") result->createAttribute("qual", a_qual);

  return result;
}
