// ANNA - Anna is Not Nothingness Anymore                                                         //
//                                                                                                //
// (c) Copyright 2005-2015 Eduardo Ramos Testillano & Francisco Ruiz Rayo                         //
//                                                                                                //
// See project site at http://redmine.teslayout.com/projects/anna-suite                           //
// See accompanying file LICENSE or copy at http://www.teslayout.com/projects/public/anna.LICENSE //


#include <time.h>
#include <ldap.h>

#include <anna/core/functions.hpp>
#include <anna/core/tracing/Logger.hpp>

#include <anna/xml/Node.hpp>

#include <anna/ldap/Search.hpp>
#include <anna/ldap/Session.hpp>

using namespace std;
using namespace anna;
using namespace anna::ldap;

Search::attribute_pool Search::st_attributes;

void Search::clear()
throw() {
  a_base.clear();
  a_scope = Scope::Base;
  clearFilter();
  a_onlyType = false,
  clearSizeLimit();
  clearAttributes();
}

void Search::clearAttributes()
throw() {
  for(attribute_iterator ii = attribute_begin(), maxii = attribute_end(); ii != maxii; ii ++)
    st_attributes.release(&attribute(ii));

  a_attributes.clear();
}

IdMessage Search::send(Session& session) const
throw() {
  IdMessage result(-1);
  LDAP* ldap = (LDAP*) session.getLDAP();
  int scope = -1;

  switch(a_scope) {
  case Scope::Base: scope = LDAP_SCOPE_BASE; break;
  case Scope::OneLevel: scope = LDAP_SCOPE_ONELEVEL; break;
  case Scope::SubTree: scope = LDAP_SCOPE_SUBTREE; break;
  case Scope::None: return result; break;
  }

  int maxi = attribute_size() + 1;
  char** attributes = NULL;

  if(maxi > 1) {
    attributes = new char* [maxi];
    int i = 0;

    for(const_attribute_iterator ii = attribute_begin(), maxii = attribute_end(); ii != maxii; ii ++)
      attributes [i ++] = const_cast <char*>(attribute(ii).c_str());

    attributes [i] = NULL;
  }

  result = ldap_search(ldap, a_base.c_str(), scope, asCString(a_filter), attributes, a_onlyType);
  LOGDEBUG(
    string msg("ldap::Search::send | Scope: ");
    msg += functions::asString(scope);
    msg += " | ";
    msg += asString();
    msg += " | Attributes: ";

  if(attributes != NULL) {
  for(int ii = 0; attributes [ii] != NULL; ii ++) {
      msg += attributes [ii];
      msg += ' ';
    }
  } else
    msg += "<null>";
    Logger::debug(msg, ANNA_FILE_LOCATION);
    );

  delete [] attributes;
  return result;
}

string Search::asString() const
throw() {
  string result("ldap::Search { ");
  result += Request::asString();
  result += " | Base: ";
  result += a_base;
  result += " | Scope: ";
  result += Scope::asText(a_scope);
  result += " | Filter: ";
  result += asText(a_filter);
  result += functions::asText(" | OnlyType: ", a_onlyType);
  result += functions::asText(" | SizeLimit: ", a_sizeLimit);
  return result += " }";
}

xml::Node* Search::asXML(xml::Node* parent) const
throw() {
  parent = Request::asXML(parent);
  parent->createAttribute("Base", a_base);
  parent->createAttribute("Scope", Scope::asText(a_scope));
  parent->createAttribute("Filter", asText(a_filter));
  parent->createAttribute("OnlyType", functions::asString(a_onlyType));
  parent->createAttribute("SizeLimit", a_sizeLimit);
  return parent;
}

